## GUI構築

# ウインドウ設定
wm title . [file rootname [file tail $::argv0]] ;# タイトルバーの表示文字列設定（スクリプト名に）
wm resizable . 1 0                              ;# サイズ変更の可否設定（横方向のみ可変に）

# 大枠ブロックフレーム生成
pack [frame .fPort] [frame .fClock] -fill x     ;# ポート指定欄と時刻表示・操作欄の2つのフレームを縦に積む

# ポート指定欄構築
package require tile ;# 標準Tkにコンボボックスが含まれていないため、同梱されることの多いTileパッケージのものを利用
set w .fPort
pack [label $w.l] [ttk::combobox $w.cb] -side left ;# ラベルとコンボボックスを横に並べる
$w.l configure -text {ポート：}                    ;# ラベルの設定
pack configure $w.cb -expand yes -fill x           ;# コンボボックスの設定（選択候補の設定は後ほど）

# 時刻表示・操作欄構築
set w .fClock
pack [label $w.l] [button $w.b] -fill x
$w.b configure -text {M5StickC_Watch 時刻合わせ実行} -command setClock -borderwidth 10

# ポート指定欄コンボボックスへの選択候補設定
set listPortDisplay {{（プルダウンで選択 or 直接入力）}}
if {([string toupper $::tcl_platform(os)] eq {WINDOWS NT}) && (![catch {package require registry}])} then {
  # Windows NTの場合（要registryパッケージ）
  
  # 利用可能シリアルポート情報取得
  #   参考コード：[Tcler's Wiki] [serial ports on Windows](https://wiki.tcl-lang.org/page/serial+ports+on+Windows) [getSerialInterfaces] by Holger Ewert 2014-07-13
  # ポート名の取得
  set listPort {}
  set RegistryReference {HKEY_LOCAL_MACHINE\HARDWARE\DEVICEMAP\SERIALCOMM}
  foreach i [registry values $RegistryReference] {lappend listPort [registry get $RegistryReference $i]}
  # FriendlyNameの取得
  set dictPort {}
  set RegistryReference {HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Enum}
  foreach key1 [registry keys $RegistryReference] {
    foreach key2 [registry keys "$RegistryReference\\$key1"] {
      foreach key3 [registry keys $RegistryReference\\$key1\\$key2] {
        catch {
          set Port [registry get "$RegistryReference\\$key1\\$key2\\$key3\\Device Parameters" PortName]
          if {$Port ni $listPort} {continue}
          dict append dictPort $Port [registry get "$RegistryReference\\$key1\\$key2\\$key3" FriendlyName]
        }
      }
    }
  }
  
  # コンボボックス表示用ポートリスト生成
  foreach Port $listPort {lappend listPortDisplay "$Port : [dict get $dictPort $Port]"}
  
} elseif {![catch {package require Usbserial}]} then {
  # AndroWishの場合
  
  # コンボボックス表示用ポートリスト生成
  foreach Port [usbserial] {lappend listPortDisplay "$Port"}
  
}
set w .fPort
$w.cb configure -values $listPortDisplay ;# コンボボックスへの選択候補設定
$w.cb current 0                          ;# 初期値指定

# GUI構築後のおまじない
#console show   ;# コマンドコンソール表示（デバッグ用）
focus -force . ;# トップレベルにフォーカス
update         ;# リフレッシュ


## デバイス時刻設定ルーチン
proc setClock {} {
  set Port [.fPort.cb get]  ;# ポート指定欄の入力文字列を取得
  set Port [lindex $Port 0] ;# 冒頭の1フレーズを抜き出し
  # ポートオープン（catchでエラーをトラップ）
  if {![catch {package require Usbserial}]} then {
    # AndroWishの場合
    set return [catch "usbserial $Port" result]
  } else {
    # その他の場合
    set return [catch "open $Port r+" result]
  }
  if {$return == 0} then {
    # ポートが開けた場合は時刻設定実行
    fconfigure $result -mode 9600,n,8,1 -translation lf
    puts $result [clock format [clock seconds] -format {%Y-%m-%d %T}]
    flush $result
    close $result
  }
}


## 表示更新
proc loop {} {
  .fClock.l configure -text "PCクロック現在値：[clock format [clock seconds] -format {%Y-%m-%d %T}]"
  update
  after 1 loop
}
loop
