// EMC_M5SticC_Watch Ver.20191121 by JO3EMC

// ライブラリ ヘッダーファイル読込
#include <M5StickC.h> // Library for M5StickC Core development kit

// グローバル変数定義

// M5StickCライブラリのRTC利用クラスで使う構造体（標準サンプルプログラムに同じ）
RTC_TimeTypeDef RTC_TimeStruct; // 時刻構造体
RTC_DateTypeDef RTC_DateStruct; // 日付構造体

// 本スケッチ独自の変数
unsigned long millisBrightened = 0;   // 表示輝度アップに時限タイマーを仕掛けるためのタイムスタンプ記録用変数
String stringIn  = "";                // シリアル通信入力文字列格納用 Stringクラス

void setup() {
  // ボタンA(HOME),B(RST)に対応するピンを入力モードに設定
  pinMode(BUTTON_A_PIN,INPUT);
  pinMode(BUTTON_B_PIN,INPUT);

  M5.begin();             // M5ライブラリ初期化
  setCpuFrequencyMhz(10); // 消費電力を抑えるため、CPU周波数を最小に設定
  M5.Lcd.setRotation(3);  // 画面の向きを横向き（HOMEボタンが左）に設定
  
  Serial.begin(9600); // シリアル通信のデータ転送レートを設定（気持ち消費電力を押さえたくて低めで）
}

void loop() {
  M5.Rtc.GetTime(&RTC_TimeStruct); // RTCから時刻を取得
  M5.Rtc.GetData(&RTC_DateStruct); // RTCから日付を取得（関数名は実装ミス？）
  
  char stringTmp[(4+2*2+2)+1+(2*3+2)+1]; // 一時処理用テンポラリー文字配列定義（「YYYY-MM-DD HH:MM:SS」＋「\0」でサイズ設定）
  // 日付画面表示
  M5.Lcd.setTextColor(RED,BLACK);    // 文字色と文字背景色設定
  M5.Lcd.setCursor(0,5);             // 描画位置設定
  M5.Lcd.setTextFont(4);             // フォント設定
  M5.Lcd.printf("%04d-%02d-%02d",RTC_DateStruct.Year,RTC_DateStruct.Month,RTC_DateStruct.Date); // 画面表示
  // 時分画面表示
  M5.Lcd.setTextColor(YELLOW,BLACK); // 文字色と文字背景色設定
  M5.Lcd.setCursor(0,30);            // 描画位置設定
  M5.Lcd.setTextFont(6);             // フォント設定
  M5.Lcd.printf("%02d:%02d",RTC_TimeStruct.Hours,RTC_TimeStruct.Minutes); // 画面表示
  // 秒画面表示
  M5.Lcd.setCursor(122,47);          // 描画位置設定
  M5.Lcd.setTextFont(4);             // フォント設定
  M5.Lcd.printf(":%02d",RTC_TimeStruct.Seconds); // 画面表示

  M5.update(); // ボタンの状態更新
  
  // 表示輝度関連処理
  unsigned long millisCurrent = millis(); // タイムスタンプ現在値取得
  if ( ((millisCurrent - millisBrightened) >= 3000) || (millisCurrent < millisBrightened) ) {
    // 輝度アップしてから指定時間(ms)以上経過しているか、タイムスタンプがオーバーフローしている場合
    M5.Axp.ScreenBreath(8);      // 液晶バックライト電圧を暗く設定
  }
  if ( M5.BtnA.isPressed() ) {
    // ボタンA(HOME)が押されている場合
    M5.Axp.ScreenBreath(12);     // 液晶バックライト電圧を明るく設定
    millisBrightened = millis(); // 輝度アップ時のタイムスタンプ値を更新
  }
  
  // 月に応じた日数設定
  char DaysInMonth = 31;
  if ( RTC_DateStruct.Month ==  2 ) {DaysInMonth = 29;}
  if ( RTC_DateStruct.Month ==  4 ) {DaysInMonth = 30;}
  if ( RTC_DateStruct.Month ==  6 ) {DaysInMonth = 30;}
  if ( RTC_DateStruct.Month ==  9 ) {DaysInMonth = 30;}
  if ( RTC_DateStruct.Month == 11 ) {DaysInMonth = 30;}
  
  // ボタンによる時刻調整処理
  if ( M5.BtnB.pressedFor(1000) ) {
    // ボタンB(RST)が押されてから指定時間(ms)以上経過している場合
    RTC_TimeStruct.Seconds = 0; // 秒を0に
    if ( M5.BtnA.isPressed() ) {
      // 同時にボタンA(HOME)が押されている場合
      RTC_TimeStruct.Minutes ++; // 分をインクリメント
      if ( RTC_TimeStruct.Minutes > 59 ) {
        // 時の繰り上げ処理
        RTC_TimeStruct.Minutes = 0;
        RTC_TimeStruct.Hours ++;
      }
      if ( RTC_TimeStruct.Hours > 23 ) {
        // 日の繰り上げ処理
        RTC_TimeStruct.Hours = 0;
        RTC_DateStruct.Date ++;
      }
      if ( RTC_DateStruct.Date > DaysInMonth ) {
        // 月の繰り上げ処理
        RTC_DateStruct.Date = 1;
        RTC_DateStruct.Month ++;
      }
      if ( RTC_DateStruct.Month > 12 ) {
        // 年の繰り上げ処理
        RTC_DateStruct.Month = 1;
        RTC_DateStruct.Year ++;
      }
      RTC_DateStruct.WeekDay = 0;      // 曜日は本スケッチでは扱わない（とりあえず日曜に設定）
      M5.Rtc.SetData(&RTC_DateStruct); // 日付更新
    }
    M5.Rtc.SetTime(&RTC_TimeStruct);   // 時刻更新
  }
  
  // シリアル通信による日時設定処理
  while (Serial.available()) {
    // シリアルポートバッファに受信データがある間繰り返し
    char charIn = (char)Serial.read(); // シリアルポートバッファから受信データを1バイト取り出し
    Serial.write(charIn);              // 取り出した文字をエコーバック
    if        ( (charIn >= '0') && (charIn <= '9') ) {
      // 数字はシリアル通信入力文字列格納用Stringに蓄積
      stringIn += charIn;
    } else if ( (charIn == ' ') || (charIn == '-') || (charIn == '/') || (charIn == ':') ) {
      // 日時表記に多用される一部の文字はスルー
    } else if ( (charIn == 0x0A) || (charIn == 0x0D) ) {
      // 改行または復帰が来たらシリアル通信入力文字列を処理して日時設定実行
      if ( stringIn.length() >= 8 ) {
        // 文字列長が8文字以上の場合は冒頭を日付情報として処理
        RTC_DateStruct.WeekDay = 0;                               // 曜日は本スケッチでは扱わない（とりあえず日曜に設定）
        RTC_DateStruct.Year    = stringIn.substring(0,4).toInt(); // 0～3文字目を整数化して年の値に設定
        RTC_DateStruct.Month   = stringIn.substring(4,6).toInt(); // 4～5文字目を整数化して月の値に設定
        RTC_DateStruct.Date    = stringIn.substring(6,8).toInt(); // 6～7文字目を整数化して日の値に設定
        if ( (RTC_DateStruct.Year < 10000)
          && (RTC_DateStruct.Month >= 1) && (RTC_DateStruct.Month <= 12)
          && (RTC_DateStruct.Date  >= 1) && (RTC_DateStruct.Date  <= DaysInMonth)
        ) {
          // 年月日の値がざっくり不自然な値でなければ、日付設定実行
          M5.Rtc.SetData(&RTC_DateStruct);
        }
        stringIn = stringIn.substring(8); // シリアル通信入力文字列から冒頭8文字を削除して前に詰める
      }
      if ( stringIn.length() >= 4 ) {
        // 文字列長が4文字以上の場合は冒頭を時刻情報として処理
        RTC_TimeStruct.Hours   = stringIn.substring(0,2).toInt(); // 0～1文字目を整数化して時の値に設定
        RTC_TimeStruct.Minutes = stringIn.substring(2,4).toInt(); // 2～3文字目を整数化して分の値に設定
        RTC_TimeStruct.Seconds = stringIn.substring(4,6).toInt(); // 4～5文字目を整数化して秒の値に設定
        if ( (RTC_TimeStruct.Hours   >= 0) && (RTC_TimeStruct.Hours   <= 23)
          && (RTC_TimeStruct.Minutes < 60)
          && (RTC_TimeStruct.Seconds < 60)
        ) {
          // 時分秒の値がざっくり不自然な値でなければ、時刻設定実行
          M5.Rtc.SetTime(&RTC_TimeStruct);
        }
      }
      stringIn = ""; // シリアル通信入力文字列をクリア
    } else {
      // その他の入力文字が来たらシリアル通信入力文字列をクリア（初期版ではスルーしていたが、不随意な設定変更が生じるバグを緩和するため一部文字を除いてクリア処理に変更）
      stringIn = ""; // シリアル通信入力文字列をクリア
    }
  }
  
  delay(100); // ループディレイ
}
